@@ -8,7 +8,7 @@ extern "C" {
 /* clp.h - Public interface to CLP.
  * This file is part of CLP, the command line parser package.
  *
- * Copyright (c) 1997-2024 Eddie Kohler, ekohler@gmail.com
+ * Copyright (c) 1997-2025 Eddie Kohler, ekohler@gmail.com
  *
  * CLP is free software. It is distributed under the GNU General Public
  * License, Version 2, or, alternatively and at your discretion, under the
@@ -327,6 +327,9 @@ int Clp_IsLong(Clp_Parser *clp, const char *long_name);
 /** @brief Test if the current option had short name @a name. */
 int Clp_IsShort(Clp_Parser *clp, int short_name);
 
+/** @brief Return a UTF-8 or normal single quote. */
+const char* Clp_QuoteChar(const Clp_Parser* clp, int isclose);
+
 #undef CLP_SENTINEL
 #ifdef __cplusplus
 }
