@@ -1,6 +1,6 @@
 /* -*- c-basic-offset: 2 -*- */
 /* gifsicle.c - gifsicle's main loop.
-   Copyright (C) 1997-2024 Eddie Kohler, ekohler@gmail.com
+   Copyright (C) 1997-2025 Eddie Kohler, ekohler@gmail.com
    This file is part of gifsicle.
 
    Gifsicle is free software. It is distributed under the GNU Public License,
@@ -202,6 +202,7 @@ static const char *output_option_types[] = {
 #define RESIZE_TOUCH_WIDTH_OPT  378
 #define RESIZE_TOUCH_HEIGHT_OPT 379
 #define LOSSY_OPT               380
+#define USE_EXACT_COLORMAP_OPT  381
 
 #define LOOP_TYPE               (Clp_ValFirstUser)
 #define DISPOSAL_TYPE           (Clp_ValFirstUser + 1)
@@ -340,6 +341,7 @@ const Clp_Option options[] = {
 
   { "unoptimize", 'U', UNOPTIMIZE_OPT, 0, Clp_Negate },
   { "use-colormap", 0, USE_COLORMAP_OPT, Clp_ValString, Clp_Negate },
+  { "use-exact-colormap", 0, USE_EXACT_COLORMAP_OPT, Clp_ValString, Clp_Negate },
 
   { "verbose", 'V', VERBOSE_OPT, 0, Clp_Negate },
   { 0, 'v', VERBOSE_OPT, 0, Clp_Negate },
@@ -455,7 +457,8 @@ set_frame_change(int kind)
    case INSERT_OPT:
     /* Define a nested frameset (or use an existing one). */
     fset = FRAME(frames, frame_spec_2).nest;
-    if (!fset) fset = new_frameset(8);
+    if (!fset)
+      fset = new_frameset(8);
     FRAME(frames, frame_spec_2).nest = fset;
 
     /* Later: Merge frames at the end of the nested frameset. */
@@ -846,9 +849,11 @@ input_stream(const char *name)
 void
 input_done(void)
 {
-  if (!input) return;
+  if (!input)
+    return;
 
-  if (verbosing) verbose_close('>');
+  if (verbosing)
+    verbose_close('>');
 
   Gif_DeleteStream(input);
   input = 0;
@@ -865,7 +870,7 @@ input_done(void)
  **/
 
 static void
-set_new_fixed_colormap(const char *name)
+set_new_fixed_colormap(const char *name, int exact)
 {
   int i;
   if (name && strcmp(name, "web") == 0) {
@@ -894,23 +899,21 @@ set_new_fixed_colormap(const char *name)
 
   } else
     def_output_data.colormap_fixed = read_colormap_file(name, 0);
+
+  def_output_data.colormap_fixed_exact = exact;
 }
 
 static void
 do_colormap_change(Gif_Stream *gfs)
 {
-  if (active_output_data.colormap_fixed || active_output_data.colormap_size > 0)
-    kc_set_gamma(active_output_data.colormap_gamma_type,
-                 active_output_data.colormap_gamma);
-
   if (active_output_data.colormap_fixed)
     colormap_stream(gfs, active_output_data.colormap_fixed,
-                    &active_output_data);
+                    &active_output_data, !active_output_data.colormap_fixed_exact);
 
   if (active_output_data.colormap_size > 0) {
     kchist kch;
     Gif_Colormap* (*adapt_func)(kchist*, Gt_OutputData*);
-    Gif_Colormap *new_cm;
+    Gif_Colormap* new_cm;
 
     /* set up the histogram */
     {
@@ -945,7 +948,7 @@ do_colormap_change(Gif_Stream *gfs)
     }
 
     new_cm = (*adapt_func)(&kch, &active_output_data);
-    colormap_stream(gfs, new_cm, &active_output_data);
+    colormap_stream(gfs, new_cm, &active_output_data, 1);
 
     Gif_DeleteColormap(new_cm);
     kchist_cleanup(&kch);
@@ -1025,6 +1028,8 @@ merge_and_write_frames(const char *outfile, int f1, int f2)
       w = active_output_data.resize_width;
       h = active_output_data.resize_height;
     }
+    kc_set_gamma(active_output_data.colormap_gamma_type,
+                 active_output_data.colormap_gamma);
     if (active_output_data.scaling != GT_SCALING_NONE)
       resize_stream(out, w, h, active_output_data.resize_flags,
                     active_output_data.scale_method,
@@ -1250,6 +1255,7 @@ initialize_def_frame(void)
   /* output defaults */
   def_output_data.output_name = 0;
 
+  def_output_data.screen_mode = 0;
   def_output_data.screen_width = -1;
   def_output_data.screen_height = -1;
   def_output_data.background.haspixel = 0;
@@ -1257,6 +1263,7 @@ initialize_def_frame(void)
 
   def_output_data.colormap_size = 0;
   def_output_data.colormap_fixed = 0;
+  def_output_data.colormap_fixed_exact = 0;
   def_output_data.colormap_algorithm = COLORMAP_DIVERSITY;
   def_output_data.dither_type = dither_none;
   def_output_data.dither_name = "none";
@@ -1288,6 +1295,7 @@ combine_output_options(void)
 
   if (CHANGED(recent, CH_LOGICAL_SCREEN)) {
     MARK_CH(output, CH_LOGICAL_SCREEN);
+    active_output_data.screen_mode = def_output_data.screen_mode;
     active_output_data.screen_width = def_output_data.screen_width;
     active_output_data.screen_height = def_output_data.screen_height;
   }
@@ -1303,6 +1311,7 @@ combine_output_options(void)
       def_output_data.colormap_fixed->refcount++;
     Gif_DeleteColormap(active_output_data.colormap_fixed);
     active_output_data.colormap_fixed = def_output_data.colormap_fixed;
+    active_output_data.colormap_fixed_exact = def_output_data.colormap_fixed_exact;
   }
   if (CHANGED(recent, CH_DITHER)) {
     MARK_CH(output, CH_DITHER);
@@ -1338,6 +1347,7 @@ combine_output_options(void)
   COMBINE_ONE_OUTPUT_OPTION(CH_MEMORY, conserve_memory);
 
   def_output_data.colormap_fixed = 0;
+  def_output_data.colormap_fixed_exact = 0;
   def_output_data.output_name = 0;
 
   active_next_output |= next_output;
@@ -1703,9 +1713,11 @@ main(int argc, char *argv[])
 
      case LOGICAL_SCREEN_OPT:
       MARK_CH(output, CH_LOGICAL_SCREEN);
-      if (clp->negated)
+      if (clp->negated) {
+        def_output_data.screen_mode = -1;
         def_output_data.screen_width = def_output_data.screen_height = 0;
-      else {
+      } else {
+        def_output_data.screen_mode = 1;
         def_output_data.screen_width = dimensions_x;
         def_output_data.screen_height = dimensions_y;
       }
@@ -1713,11 +1725,13 @@ main(int argc, char *argv[])
 
      case SAME_LOGICAL_SCREEN_OPT:
       MARK_CH(output, CH_LOGICAL_SCREEN);
+      def_output_data.screen_mode = 0;
       def_output_data.screen_width = def_output_data.screen_height = -1;
       break;
 
      case CROP_OPT:
-      if (clp->negated) goto no_crop;
+      if (clp->negated)
+        goto no_crop;
       MARK_CH(frame, CH_CROP);
       {
         Gt_Crop *crop = copy_crop(def_frame.crop);
@@ -1930,7 +1944,7 @@ main(int argc, char *argv[])
      case GRAY_OPT:
       MARK_CH(output, CH_USE_COLORMAP);
       Gif_DeleteColormap(def_output_data.colormap_fixed);
-      set_new_fixed_colormap("gray");
+      set_new_fixed_colormap("gray", 0);
       break;
 
     case USE_COLORMAP_OPT:
@@ -1939,7 +1953,16 @@ main(int argc, char *argv[])
       if (clp->negated)
         def_output_data.colormap_fixed = 0;
       else
-        set_new_fixed_colormap(clp->vstr);
+        set_new_fixed_colormap(clp->vstr, 0);
+      break;
+
+    case USE_EXACT_COLORMAP_OPT:
+      MARK_CH(output, CH_USE_COLORMAP);
+      Gif_DeleteColormap(def_output_data.colormap_fixed);
+      if (clp->negated)
+        def_output_data.colormap_fixed = 0;
+      else
+        set_new_fixed_colormap(clp->vstr, 1);
       break;
 
      case COLORMAP_ALGORITHM_OPT:
@@ -1969,26 +1992,46 @@ main(int argc, char *argv[])
     }
 
     case GAMMA_OPT: {
-#if HAVE_POW
-      char* ends;
-      MARK_CH(output, CH_GAMMA);
       if (clp->negated) {
+        MARK_CH(output, CH_GAMMA);
         def_output_data.colormap_gamma_type = KC_GAMMA_NUMERIC;
-        def_output_data.colormap_gamma = 1;
-      } else if (strcmp(clp->val.s, "sRGB") == 0
-                 || strcmp(clp->val.s, "srgb") == 0)
+        def_output_data.colormap_gamma = 1.0;
+        break;
+      }
+      if (strlen(clp->val.s) == 4
+          && tolower((unsigned char) clp->val.s[0]) == 's'
+          && tolower((unsigned char) clp->val.s[1]) == 'r'
+          && tolower((unsigned char) clp->val.s[2]) == 'g'
+          && tolower((unsigned char) clp->val.s[3]) == 'b') {
+        MARK_CH(output, CH_GAMMA);
         def_output_data.colormap_gamma_type = KC_GAMMA_SRGB;
-      else {
+        break;
+      }
+#if HAVE_CBRTF
+      if (strlen(clp->val.s) == 5
+          && tolower((unsigned char) clp->val.s[0]) == 'o'
+          && tolower((unsigned char) clp->val.s[1]) == 'k'
+          && tolower((unsigned char) clp->val.s[2]) == 'l'
+          && tolower((unsigned char) clp->val.s[3]) == 'a'
+          && tolower((unsigned char) clp->val.s[4]) == 'b') {
+        MARK_CH(output, CH_GAMMA);
+        def_output_data.colormap_gamma_type = KC_GAMMA_OKLAB;
+        break;
+      }
+#endif
+#if HAVE_POW
+      {
+        char* ends;
         double gamma = strtod(clp->val.s, &ends);
         if (*clp->val.s && !*ends && !isspace((unsigned char) *clp->val.s)) {
+          MARK_CH(output, CH_GAMMA);
           def_output_data.colormap_gamma_type = KC_GAMMA_NUMERIC;
           def_output_data.colormap_gamma = gamma;
-        } else
-          Clp_OptionError(clp, "%O should be a number or %<srgb%>");
+          break;
+        }
       }
-#else
-      Clp_OptionError(clp, "this version of Gifsicle does not support %O");
 #endif
+      Clp_OptionError(clp, "%O not supported");
       break;
     }
 
@@ -2126,7 +2169,7 @@ main(int argc, char *argv[])
 #else
       printf("LCDF Gifsicle %s\n", VERSION);
 #endif
-      printf("Copyright (C) 1997-2024 Eddie Kohler\n\
+      printf("Copyright (C) 1997-2025 Eddie Kohler\n\
 This is free software; see the source for copying conditions.\n\
 There is NO warranty, not even for merchantability or fitness for a\n\
 particular purpose.\n");
@@ -2134,7 +2177,7 @@ particular purpose.\n");
       break;
 
      case HELP_OPT:
-      usage();
+      usage(clp);
       exit(EXIT_OK);
       break;
 
@@ -2160,7 +2203,7 @@ particular purpose.\n");
 
      bad_option:
      case Clp_BadOption:
-      short_usage();
+      short_usage(clp);
       exit(EXIT_USER_ERR);
       break;
 
