@@ -1,5 +1,5 @@
 /* support.c - Support functions for gifsicle.
-   Copyright (C) 1997-2024 Eddie Kohler, ekohler@gmail.com
+   Copyright (C) 1997-2025 Eddie Kohler, ekohler@gmail.com
    This file is part of gifsicle.
 
    Gifsicle is free software. It is distributed under the GNU Public License,
@@ -122,120 +122,130 @@ clp_error_handler(Clp_Parser *clp, const char *message)
 
 
 void
-short_usage(void)
+short_usage(Clp_Parser* clp)
 {
   fprintf(stderr, "Usage: %s [OPTION | FILE | FRAME]...\n\
-Try '%s --help' for more information.\n",
-          program_name, program_name);
+Try %s%s --help%s for more information.\n",
+          program_name, Clp_QuoteChar(clp, 0), program_name, Clp_QuoteChar(clp, 1));
 }
 
 
 void
-usage(void)
+usage(Clp_Parser* clp)
 {
+  const char* q0 = Clp_QuoteChar(clp, 0);
+  const char* q1 = Clp_QuoteChar(clp, 1);
   printf("\
-'Gifsicle' manipulates GIF images. Its most common uses include combining\n\
+Gifsicle manipulates GIF images. Its most common uses include combining\n\
 single images into animations, adding transparency, optimizing animations for\n\
 space, and printing information about GIFs.\n\
 \n\
 Usage: %s [OPTION | FILE | FRAME]...\n\n", program_name);
   printf("\
-Mode options: at most one, before any filenames.\n\
-  -m, --merge                   Merge mode: combine inputs, write stdout.\n\
+Mode options (at most one, before any filenames):\n\
+  -m, --merge                   Merge mode: combine inputs, write stdout\n\
   -b, --batch                   Batch mode: modify inputs, write back to\n\
-                                same filenames.\n\
+                                same filenames\n\
   -e, --explode                 Explode mode: write N files for each input,\n\
-                                one per frame, to 'input.frame-number'.\n\
-  -E, --explode-by-name         Explode mode, but write 'input.name'.\n\n");
+                                one per frame, to %sinput.frame-number%s\n\
+  -E, --explode-by-name         Explode mode, but write %sinput.name%s\n\n",
+    q0, q1, q0, q1);
   printf("\
-General options: Also --no-OPTION for info and verbose.\n\
-  -I, --info                    Print info about input GIFs. Two -I's means\n\
-                                normal output is not suppressed.\n\
-      --color-info, --cinfo     --info plus colormap details.\n\
-      --extension-info, --xinfo --info plus extension details.\n\
-      --size-info, --sinfo      --info plus compression information.\n\
-  -V, --verbose                 Prints progress information.\n");
+General options (also --no-OPTION for info and verbose):\n\
+  -I, --info                    Print info about input GIFs\n\
+  -I -I                         Print info to standard error; do not suppress\n\
+                                normal output\n\
+  --color-info, --cinfo         --info plus colormap details\n\
+  --extension-info, --xinfo     --info plus extension details\n\
+  --size-info, --sinfo          --info plus compression information\n\
+  -V, --verbose                 Print progress (files read and written)\n");
   printf("\
-  -h, --help                    Print this message and exit.\n\
-      --version                 Print version number and exit.\n\
-  -o, --output FILE             Write output to FILE.\n\
-  -w, --no-warnings             Don't report warnings.\n\
-      --no-ignore-errors        Quit on very erroneous input GIFs.\n\
-      --conserve-memory         Conserve memory at the expense of speed.\n\
-      --multifile               Support concatenated GIF files.\n\
-\n");
+  -h, --help                    Print this message and exit\n\
+  --version                     Print version number and exit\n\
+  -o, --output FILE             Write output to FILE\n\
+  -w, --no-warnings             Don%st report warnings\n\
+  --no-ignore-errors            Quit on very erroneous input GIFs\n\
+  --conserve-memory             Conserve memory at the expense of speed\n\
+  --nextfile, --multifile       Support concatenated GIF files\n\n",
+    q1);
   printf("\
 Frame selections:               #num, #num1-num2, #num1-, #name\n\
 \n\
 Frame change options:\n\
-  --delete FRAMES               Delete FRAMES from input.\n\
-  --insert-before FRAME GIFS    Insert GIFS before FRAMES in input.\n\
-  --append GIFS                 Append GIFS to input.\n\
-  --replace FRAMES GIFS         Replace FRAMES with GIFS in input.\n\
-  --done                        Done with frame changes.\n\n");
+  --delete FRAMES               Delete FRAMES from input\n\
+  --insert-before FRAME GIFS    Insert GIFS before FRAMES in input\n\
+  --append GIFS                 Append GIFS to input\n\
+  --replace FRAMES GIFS         Replace FRAMES with GIFS in input\n\
+  --done                        Done with frame changes\n\n");
   printf("\
-Image options: Also --no-OPTION and --same-OPTION.\n\
-  -B, --background COL          Make COL the background color.\n\
-      --crop X,Y+WxH, --crop X,Y-X2,Y2\n\
-                                Crop the image.\n\
-      --crop-transparency       Crop transparent borders off the image.\n\
-      --flip-horizontal, --flip-vertical\n\
-                                Flip the image.\n");
+Image options (also --no-OPTION and --same-OPTION):\n\
+  -B, --background COL          Make COL the background color\n\
+  --crop X,Y+WxH, --crop X,Y-X2,Y2\n\
+                                Crop the image\n\
+  --crop-transparency           Crop transparent borders off the image\n\
+  --flip-horizontal, --flip-vertical\n\
+                                Flip the image\n");
   printf("\
-  -i, --interlace               Turn on interlacing.\n\
-  -S, --logical-screen WxH      Set logical screen to WxH.\n\
-  -p, --position X,Y            Set frame position to (X,Y).\n\
-      --rotate-90, --rotate-180, --rotate-270, --no-rotate\n\
-                                Rotate the image.\n\
-  -t, --transparent COL         Make COL transparent.\n\n");
+  -i, --interlace               Turn on interlacing\n\
+  -S, --logical-screen WxH      Set logical screen to WxH\n\
+  -p, --position X,Y            Set frame position to (X,Y)\n\
+  --rotate-90, --rotate-180, --rotate-270, --no-rotate\n\
+                                Rotate the image\n\
+  -t, --transparent COL         Make COL transparent\n\n");
   printf("\
 Extension options:\n\
-      --app-extension N D       Add an app extension named N with data D.\n\
-  -c, --comment TEXT            Add a comment before the next frame.\n\
-      --extension N D           Add an extension number N with data D.\n\
-  -n, --name TEXT               Set next frame's name.\n\
-      --no-comments, --no-names, --no-extensions\n\
-                                Remove comments (names, extensions) from input.\n");
+  --app-extension N D           Add an app extension named N with data D\n\
+  -c, --comment TEXT            Add a comment before the next frame\n\
+  --extension N D           Add an extension number N with data D\n\
+  -n, --name TEXT               Set next frame%ss name\n\
+  --no-comments, --no-names, --no-extensions\n\
+                                Remove comments (names, extensions) from input\n",
+    q1);
   printf("\
-Animation options: Also --no-OPTION and --same-OPTION.\n\
-  -d, --delay TIME              Set frame delay to TIME (in 1/100sec).\n\
-  -D, --disposal METHOD         Set frame disposal to METHOD.\n\
-  -l, --loopcount[=N]           Set loop extension to N (default forever).\n\
-  -O, --optimize[=LEVEL]        Optimize output GIFs.\n\
-  -U, --unoptimize              Unoptimize input GIFs.\n");
+Animation options (also --no-OPTION and --same-OPTION):\n\
+  -d, --delay TIME              Set frame delay to TIME (in 1/100sec)\n\
+  -D, --disposal METHOD         Set frame disposal to METHOD\n\
+  -l, --loopcount[=N]           Set loop extension to N (default forever)\n\
+  -O, --optimize[=LEVEL]        Optimize output GIFs\n\
+  -U, --unoptimize              Unoptimize input GIFs\n");
 #if ENABLE_THREADS
   printf("\
-  -j, --threads[=THREADS]       Use multiple threads to improve speed.\n");
+  -j, --threads[=THREADS]       Use multiple threads to improve speed\n");
 #endif
   printf("\n\
-Whole-GIF options: Also --no-OPTION.\n\
-      --careful                 Write larger GIFs that avoid bugs in other\n\
-                                programs.\n\
-      --change-color COL1 COL2  Change COL1 to COL2 throughout.\n\
-  -k, --colors N                Reduce the number of colors to N.\n\
-      --color-method METHOD     Set method for choosing reduced colors.\n\
-  -f, --dither                  Dither image after changing colormap.\n");
+Whole-GIF options (also --no-OPTION):\n\
+  --careful                     Write larger GIFs that avoid bugs in other\n\
+                                programs\n\
+  --change-color COL1 COL2      Change COL1 to COL2 throughout\n\
+  -k, --colors N                Reduce the number of colors to N\n\
+  --color-method METHOD         Set method for choosing reduced colors\n\
+  -f, --dither                  Dither image after changing colormap\n");
 #if HAVE_POW
   printf("\
-      --gamma G                 Set gamma for color reduction [2.2].\n");
+  --gamma G                     Set gamma for color reduction [2.2]\n");
 #endif
   printf("\
-      --lossy[=LOSSINESS]       Alter image colors to shrink output file size\n\
-                                at the cost of artifacts and noise.\n\
-      --resize WxH              Resize the output GIF to WxH.\n\
-      --resize-width W          Resize to width W and proportional height.\n\
-      --resize-height H         Resize to height H and proportional width.\n\
-      --resize-fit WxH          Resize if necessary to fit within WxH.\n");
+  --lossy[=LOSSINESS]           Alter image colors to shrink output file size\n\
+                                at the cost of artifacts and noise\n\
+  --resize WxH                  Resize the output GIF to WxH\n\
+  --scale XFACTOR[xYFACTOR]     Scale the output GIF by XFACTORxYFACTOR\n\
+  --resize-width W              Resize to width W and proportional height\n\
+  --resize-height H             Resize to height H and proportional width\n\
+  --resize-fit WxH              Shrink to fit WxH, preserving aspect ratio\n\
+  --resize-touch WxH            Resize to touch WxH, preserving aspect ratio\n\
+  --resize-geometry GEOM        Resize to a geometry specification\n");
   printf("\
-      --scale XFACTOR[xYFACTOR] Scale the output GIF by XFACTORxYFACTOR.\n\
-      --resize-method METHOD    Set resizing method.\n\
-      --resize-colors N         Resize can add new colors up to N.\n\
-      --transform-colormap CMD  Transform each output colormap by shell CMD.\n\
-      --use-colormap CMAP       Set output GIF's colormap to CMAP, which can\n\
-                                be 'web', 'gray', 'bw', or a GIF file.\n\n");
+  --resize-method METHOD        Set resizing method\n\
+  --resize-colors N             Resize can add new colors up to N\n\
+  --transform-colormap CMD      Transform each output colormap by shell CMD\n\
+  --use-colormap CMAP           Output GIF uses colors from CMAP, which can\n\
+                                be %sweb%s, %sgray%s, %sbw%s, or a GIF file\n\
+  --use-exact-colormap CMAP     Same, but use CMAP as is (not a subset)\n\
+  --gray                        Same as --use-colormap gray\n\n",
+    q1, q0, q1, q0, q1, q0);
   printf("\
 Report bugs to <ekohler@gmail.com>.\n\
-Too much information? Try '%s --help | less'.\n", program_name);
+Too much information? Try %s%s --help | less%s.\n", q0, program_name, q1);
 #ifdef GIF_UNGIF
   printf("\
 This version of Gifsicle writes uncompressed GIFs, which can be far larger\n\
@@ -1247,21 +1257,8 @@ fix_total_crop(Gif_Stream *dest, Gif_Image *srci, int merger_index)
 
 
 static void
-handle_screen(Gif_Stream *dest, uint16_t width, uint16_t height)
+handle_flip_and_rotate(Gif_Image* desti, Gt_Frame* fr)
 {
-    /* Set the screen width & height, if the current input width and height are
-       larger */
-    if (dest->screen_width < width)
-        dest->screen_width = width;
-    if (dest->screen_height < height)
-        dest->screen_height = height;
-}
-
-static void
-handle_flip_and_screen(Gif_Stream* dest, Gif_Image* desti, Gt_Frame* fr)
-{
-    Gif_Stream* gfs = fr->stream;
-
     desti->left += fr->left_offset;
     desti->top += fr->top_offset;
 
@@ -1280,12 +1277,6 @@ handle_flip_and_screen(Gif_Stream* dest, Gif_Image* desti, Gt_Frame* fr)
 
     desti->left -= fr->left_offset;
     desti->top -= fr->top_offset;
-
-    /* handle screen size, which might have height & width exchanged */
-    if (fr->rotation == 1 || fr->rotation == 3)
-        handle_screen(dest, gfs->screen_height, gfs->screen_width);
-    else
-        handle_screen(dest, gfs->screen_width, gfs->screen_height);
 }
 
 static void
@@ -1350,7 +1341,7 @@ analyze_crop(int nmerger, Gt_Crop* crop, int compress_immediately)
 
     for (i = 0;
          i < nmerger && (l > have_l || t > have_t || r < have_r || b < have_b);
-         ++i)
+         ++i) {
       if (merger[i]->crop == crop) {
         Gt_Frame *fr = merger[i];
         Gif_Image *srci = fr->image;
@@ -1420,6 +1411,7 @@ analyze_crop(int nmerger, Gt_Crop* crop, int compress_immediately)
             b = bb;
         }
       }
+    }
 
     if (t > b) {
       crop->w = crop->h = 0;
@@ -1428,6 +1420,8 @@ analyze_crop(int nmerger, Gt_Crop* crop, int compress_immediately)
       crop->y = t;
       crop->w = r - l;
       crop->h = b - t;
+      crop->left_offset = crop->x;
+      crop->top_offset = crop->y;
     }
   }
 }
@@ -1534,12 +1528,14 @@ merge_frame_interval(Gt_Frameset *fset, int f1, int f2,
     all_same_compressed_ok = 0;
 
   /* analyze crops */
-  for (i = 0; i < nmerger; i++)
+  for (i = 0; i < nmerger; i++) {
     if (merger[i]->crop)
       merger[i]->crop->ready = all_same_compressed_ok = 0;
-  for (i = 0; i < nmerger; i++)
+  }
+  for (i = 0; i < nmerger; i++) {
     if (merger[i]->crop && !merger[i]->crop->ready)
       analyze_crop(nmerger, merger[i]->crop, compress_immediately);
+  }
 
   /* mark used colors */
   for (i = 0; i < nmerger; ++i) {
@@ -1602,11 +1598,10 @@ merge_frame_interval(Gt_Frameset *fset, int f1, int f2,
 
     srci->transparent = old_transp; /* restore real transparent value */
 
-    /* Flipping and rotating, and also setting the screen size */
-    if (fr->flip_horizontal || fr->flip_vertical || fr->rotation)
-        handle_flip_and_screen(dest, desti, fr);
-    else
-        handle_screen(dest, fr->stream->screen_width, fr->stream->screen_height);
+    /* Flipping and rotating */
+    if (fr->flip_horizontal || fr->flip_vertical || fr->rotation) {
+      handle_flip_and_rotate(desti, fr);
+    }
 
     /* Names and comments */
     if (fr->name || fr->no_name) {
@@ -1627,8 +1622,9 @@ merge_frame_interval(Gt_Frameset *fset, int f1, int f2,
       fr->comment = 0;
     }
 
-    if (fr->interlacing >= 0)
+    if (fr->interlacing >= 0) {
       desti->interlace = fr->interlacing;
+    }
     if (fr->left >= 0) {
       int left = fr->left + (fr->position_is_offset ? desti->left : 0);
       if (left + desti->width > 65535) {
@@ -1646,10 +1642,43 @@ merge_frame_interval(Gt_Frameset *fset, int f1, int f2,
       desti->top = top;
     }
 
-    if (fr->delay >= 0)
+    if (fr->delay >= 0) {
       desti->delay = fr->delay;
-    if (fr->disposal >= 0)
+    }
+    if (fr->disposal >= 0) {
       desti->disposal = fr->disposal;
+    }
+
+    /* logical screen */
+    if (output_data->screen_mode <= 0) {
+      int w, h;
+      if (output_data->screen_mode < 0
+          || fr->crop
+          || ((fr->left >= 0 || fr->top >= 0) && !fr->position_is_offset)) {
+        w = desti->left + desti->width;
+        h = desti->top + desti->height;
+      } else {
+        if (fr->rotation == 1 || fr->rotation == 3) {
+          w = fr->stream->screen_height;
+          h = fr->stream->screen_width;
+        } else {
+          w = fr->stream->screen_width;
+          h = fr->stream->screen_height;
+        }
+        if (fr->left >= 0) {
+          w += fr->left;
+        }
+        if (fr->top >= 0) {
+          h += fr->top;
+        }
+      }
+      if (w > dest->screen_width) {
+        dest->screen_width = w;
+      }
+      if (h > dest->screen_height) {
+        dest->screen_height = h;
+      }
+    }
 
     /* compress immediately if possible to save on memory */
     if (desti->img) {
@@ -1665,18 +1694,20 @@ merge_frame_interval(Gt_Frameset *fset, int f1, int f2,
 
    merge_frame_done:
     /* 6/17/02 store information about image's background */
-    if (fr->stream->images[0]->transparent >= 0)
+    if (fr->stream->images[0]->transparent >= 0) {
         fr->transparent.haspixel = 2;
-    else if (fr->stream->global
-             && fr->stream->background < fr->stream->global->ncol) {
+    } else if (fr->stream->global
+               && fr->stream->background < fr->stream->global->ncol) {
         fr->transparent = fr->stream->global->col[fr->stream->background];
         fr->transparent.haspixel = 1;
-    } else
+    } else {
         fr->transparent.haspixel = 0;
+    }
 
     /* Destroy the copied, cropped image if necessary */
-    if (fr->crop)
+    if (fr->crop) {
       Gif_DeleteImage(srci);
+    }
 
     /* if we can, delete the image's data right now to save memory */
     srci = fr->image;
@@ -1696,28 +1727,10 @@ merge_frame_interval(Gt_Frameset *fset, int f1, int f2,
   }
   /** END MERGE LOOP **/
 
-  /* Cropping the whole output? Reset logical screen */
-  if (merger[0]->crop && merger[0]->crop == merger[nmerger - 1]->crop) {
-    /* 13.May.2008: Set the logical screen to the cropped dimensions */
-    /* 18.May.2008: Unless --crop-transparency is on */
-    Gt_Crop* crop = merger[0]->crop;
-    if (crop->transparent_edges)
-        dest->screen_width = dest->screen_height = 0;
-    else if (merger[0]->rotation == 1 || merger[0]->rotation == 3) {
-        dest->screen_width = (crop->h > 0 ? crop->h : 0);
-        dest->screen_height = (crop->w > 0 ? crop->w : 0);
-    } else {
-        dest->screen_width = (crop->w > 0 ? crop->w : 0);
-        dest->screen_height = (crop->h > 0 ? crop->h : 0);
-    }
-  }
-
-  /* Set the logical screen from the user's preferences */
-  if (output_data->screen_width >= 0)
+  if (output_data->screen_mode == 1) {
     dest->screen_width = output_data->screen_width;
-  if (output_data->screen_height >= 0)
     dest->screen_height = output_data->screen_height;
-  Gif_CalculateScreenSize(dest, 0);
+  }
 
   /* Find the background color in the colormap, or add it if we can */
   set_background(dest, output_data);
