@@ -1,5 +1,5 @@
 /* xform.c - Image transformation functions for gifsicle.
-   Copyright (C) 1997-2024 Eddie Kohler, ekohler@gmail.com
+   Copyright (C) 1997-2025 Eddie Kohler, ekohler@gmail.com
    This file is part of gifsicle.
 
    Gifsicle is free software. It is distributed under the GNU Public License,
@@ -215,11 +215,6 @@ pipe_color_transformer(Gif_Colormap *gfcm, void *thunk)
 }
 
 
-
-/*****
- * crop image; returns true if the image exists
- **/
-
 void
 combine_crop(Gt_Crop* dstcrop, const Gt_Crop* srccrop, const Gif_Image* gfi)
 {
@@ -244,6 +239,11 @@ combine_crop(Gt_Crop* dstcrop, const Gt_Crop* srccrop, const Gif_Image* gfi)
     }
 }
 
+
+/*****
+ * crop image; returns true if the image exists
+ **/
+
 int
 crop_image(Gif_Image* gfi, Gt_Frame* fr, int preserve_total_crop)
 {
@@ -325,6 +325,7 @@ rotate_image(Gif_Image* gfi, Gt_Frame* fr, int rotation)
   int x, y;
   int width = gfi->width;
   int height = gfi->height;
+  int ntop, nleft, nbottom, nright;
   uint8_t **img = gfi->img;
   uint8_t *new_data = Gif_NewArray(uint8_t, (unsigned) width * (unsigned) height);
   uint8_t *trav = new_data;
@@ -336,32 +337,52 @@ rotate_image(Gif_Image* gfi, Gt_Frame* fr, int rotation)
     for (x = 0; x < width; x++)
       for (y = height - 1; y >= 0; y--)
         *trav++ = img[y][x];
-    x = gfi->left;
-    gfi->left = fr->stream->screen_height - (gfi->top + height);
-    gfi->top = x;
+  } else {
+    for (x = width - 1; x >= 0; x--)
+      for (y = 0; y < height; y++)
+        *trav++ = img[y][x];
+  }
+
+  if (rotation == 1) {
+    ntop = gfi->left;
+    nleft = fr->stream->screen_height - (gfi->top + height);
     if (fr->crop) {
         x = fr->left_offset;
         fr->left_offset = fr->stream->screen_height - (fr->top_offset + fr->crop->h);
         fr->top_offset = x;
     }
-
   } else {
-    for (x = width - 1; x >= 0; x--)
-      for (y = 0; y < height; y++)
-        *trav++ = img[y][x];
-    y = gfi->top;
-    gfi->top = fr->stream->screen_width - (gfi->left + width);
-    gfi->left = y;
+    ntop = fr->stream->screen_width - (gfi->left + width);
+    nleft = gfi->top;
     if (fr->crop) {
         y = fr->top_offset;
         fr->top_offset = fr->stream->screen_width - (fr->left_offset + fr->crop->w);
         fr->left_offset = y;
     }
   }
+  nbottom = ntop + width;
+  nright = nleft + height;
+
+  if (nbottom <= 0 || nright <= 0) {
+    ntop = nleft = nbottom = nright = 0;
+  } else if (ntop < 0 || nleft < 0) {
+    int nwidth = nright - (nleft < 0 ? 0 : nleft);
+    uint8_t *xdata;
+    xdata = Gif_NewArray(uint8_t, (unsigned) (nright - nleft) * (unsigned) (nbottom - ntop));
+    for (y = ntop < 0 ? -ntop : 0; y < width; ++y) {
+        memcpy(xdata + y * nwidth, new_data + y * height + (nleft < 0 ? -nleft : 0), nwidth);
+    }
+    Gif_DeleteArray(new_data);
+    new_data = xdata;
+    ntop = ntop < 0 ? 0 : ntop;
+    nleft = nleft < 0 ? 0 : nleft;
+  }
 
   Gif_ReleaseUncompressedImage(gfi);
-  gfi->width = height;
-  gfi->height = width;
+  gfi->width = nright - nleft;
+  gfi->height = nbottom - ntop;
+  gfi->left = nleft;
+  gfi->top = ntop;
   Gif_SetUncompressedImage(gfi, new_data, Gif_Free, 0);
 }
 
@@ -482,8 +503,7 @@ static void ksscreen_init(ksscreen* kss, Gif_Stream* gfs, int sw, int sh) {
     kss->data = Gif_NewArray(scale_color, sz);
     if ((gfs->nimages == 0 || gfs->images[0]->transparent < 0)
         && gfs->global && gfs->background < gfs->global->ncol) {
-        kcolor k = kc_makegfcg(&gfs->global->col[gfs->background]);
-        kss->bg = sc_makekc(&k);
+        kss->bg = sc_makekc(kc_makegfcg(&gfs->global->col[gfs->background]));
     } else
         sc_clear(&kss->bg);
     for (i = 0; i != sz; ++i)
@@ -517,7 +537,7 @@ static void ksscreen_apply(ksscreen* kss, const Gif_Image* gfi,
         scale_color* lineout = &kss->data[y * kss->width + gfi->left];
         for (x = 0; x != gfi->width; ++x)
             if (linein[x] != gfi->transparent)
-                lineout[x] = sc_makekc(&ks[linein[x]]);
+                lineout[x] = sc_makekc(ks[linein[x]]);
     }
 }
 
@@ -726,7 +746,7 @@ static int scale_image_add_colors(scale_context* sctx, Gif_Image* gfo) {
         if (chosen >= kch.n || div.min_dist[chosen] <= sctx->max_desired_dist)
             break;
         kcdiversity_choose(&div, chosen, 0);
-        gfc = kc_togfcg(&kch.h[chosen].ka.k);
+        gfc = kc_togfcg(kch.h[chosen].ka.k);
         Gif_AddColor(gfcm, &gfc, gfcm->ncol);
         kd3_add8g(sctx->kd3, gfc.gfc_red, gfc.gfc_green, gfc.gfc_blue);
         ++nadded;
@@ -752,10 +772,10 @@ static void scale_image_complete(scale_context* sctx, Gif_Image* gfo) {
                                          + gfo->left];
         for (xo = 0; xo != gfo->width; ++xo)
             if (xscr[xo].a[3]) {
-                data[xo] = kd3_closest_transformed(sctx->kd3, &xscr[xo].k, &dist);
+                data[xo] = kd3_closest_transformed(sctx->kd3, xscr[xo].k, &dist);
                 /* maybe previous color is actually closer to desired */
                 if (transparent >= 0 && oscr[xo].a[3]) {
-                    dist2 = kc_distance(&oscr[xo].k, &xscr[xo].k);
+                    dist2 = kc_distance(oscr[xo].k, xscr[xo].k);
                     if (dist2 <= dist) {
                         data[xo] = transparent;
                         dist = dist2;
